import mapboxgl from 'mapbox-gl';
import { generateRandomString } from '../../utils/index';
export class CoreoMapboxComponent {
    constructor() {
        this.limit = 10000;
        this.showMapControls = true;
        this.pointStyle = 'circle';
        this.mapStyle = 'mapbox-light';
        this.mapboxToken = 'pk.eyJ1IjoibmF0YXBwIiwiYSI6ImNqa2poMDFxMzA2ZnMzbG1nczlyMmtkanQifQ.AWjoe6A1k08Ba9-q0ChUEQ';
        this.enableCtrl = false;
        this.recordData = [];
        this.iconData = [];
        this.syncError = false;
        this.asyncError = false;
        this.mapUUID = generateRandomString(16);
        this.showCtrlBackdrop = false;
        this.showCtrlBackdropTimeout = false;
        this.lastZoomLevel = 0;
        this.ctrlBackdropHandler = (event) => {
            if (this.enableCtrl) {
                if (event.metaKey || event.ctrlKey) {
                    event.preventDefault();
                    this.showCtrlBackdrop = false;
                    this.map.scrollZoom.enable();
                    this.map.scrollZoom.onWheel(event);
                    this.map.scrollZoom.disable();
                }
            }
        };
    }
    onPointClickedHandler(clickedFeature, sourceIndex) {
        const dataToEmit = {
            id: clickedFeature.properties.id,
            geometry: clickedFeature.geometry,
            sourceIndex: sourceIndex
        };
        this.onPointClicked.emit(dataToEmit);
    }
    resizeMap() {
        this.map.resize();
    }
    componentWillLoad() {
        mapboxgl.accessToken = 'pk.eyJ1IjoibmF0YXBwIiwiYSI6ImNqa2poMDFxMzA2ZnMzbG1nczlyMmtkanQifQ.AWjoe6A1k08Ba9-q0ChUEQ';
    }
    obtainMapStyleURL(url) {
        switch (url) {
            case 'mapbox-light': return 'mapbox://styles/mapbox/light-v9';
            case 'mapbox-dark': return 'mapbox://styles/mapbox/dark-v9';
            case 'mapbox-streets': return 'mapbox://styles/mapbox/streets-v10';
            case 'mapbox-outdoors': return 'mapbox://styles/mapbox/outdoors-v10';
            case 'mapbox-satellite': return 'mapbox://styles/mapbox/satellite-v9';
            case 'mapbox-staellite-streets': return 'mapbox://styles/mapbox/satellite-streets-v10';
            default: return url;
        }
    }
    componentDidLoad() {
        this.map = new mapboxgl.Map({
            container: this.mapUUID,
            style: this.obtainMapStyleURL(this.mapStyle),
            center: [-5, 55],
            zoom: 3.8,
            scrollZoom: !this.enableCtrl
        });
        if (this.apiCtrl) {
            try {
                this.apiCtrl.create().then(api => {
                    this.apiService = api;
                    this.sources.map((_source, index) => {
                        this.loadRecordData(index);
                    });
                });
            }
            catch (error) {
                console.error("SyncError: Coreo API error:\n", error);
                this.syncError = true;
            }
        }
        else {
            console.error("SyncError: Coreo API not found");
            this.syncError = true;
        }
    }
    loadRecordData(sourceIndex) {
        const source = this.sources[sourceIndex];
        const idQuery = source.recordId ? `id: ${source.recordId}` : '';
        const surveyQuery = source.surveyId ? `, surveyId: ${source.surveyId}` : '';
        const parentIdQuery = source.parentId ? `, parentId: ${source.parentId}` : '';
        const verificationStateQuery = source.verified ? `, state: 5` : '';
        let timeframeQuery = '';
        if (source.timeframe) {
            const currentDate = new Date();
            let timeframeQueryDate = new Date();
            timeframeQueryDate.setDate(currentDate.getDate() - source.timeframe);
            timeframeQuery = `, createdAt:{gte:"${timeframeQueryDate}"}`;
        }
        const where = `where:{ ${idQuery} ${surveyQuery} ${parentIdQuery} ${timeframeQuery} ${verificationStateQuery} }`;
        let extraAttributes = '', extraRecord = '';
        ;
        if (source.iconSourceType === 'survey' && source.surveyId) {
            extraAttributes = `attributes(where:{surveyId: ${source.surveyId}, path: "${source.iconSource}"}){
        id,
        type,
        collection{
          items{
            key
            image{
              url
            }
          }
        }
      }`;
            extraRecord = `
        iconSource: data(path: "${source.iconSource}")
        imageUrl
      `;
        }
        const query = `
            project(id: ${this.projectId}) {
                records(${where}, limit: ${this.limit}) {
                    id
                    geometry {
                        coordinates
                    }
                    ${extraRecord}
                }
                ${extraAttributes}
            }
        `;
        this.apiService.graphql(query).then(response => {
            if (response.data && response.data.project && response.data.project.records) {
                this.recordData[sourceIndex] = response.data.project.records.filter((singleRecord) => {
                    if (singleRecord.geometry && singleRecord.geometry.coordinates && singleRecord.id) {
                        return true;
                    }
                    else {
                        return false;
                    }
                });
            }
            else {
                console.error("AsyncError: Coreo API Error when obtaining record data: obtained data is malformed.", response.data);
                this.asyncError = true;
            }
            const attribute = response.data && response.data.project && response.data.project.attributes && response.data.project.attributes[0];
            if (attribute) {
                if (attribute.type === 'media') {
                    this.iconData[sourceIndex] = this.recordData[sourceIndex].reduce((acc, record) => {
                        let recordIconSource = `record-${record.id}-icon`;
                        acc[recordIconSource] = record.imageUrl || 'https://i.imgur.com/MK4NUzI.png';
                        record.iconSource = recordIconSource;
                        return acc;
                    }, {});
                }
                else if (attribute.type === 'select') {
                    this.iconData[sourceIndex] = attribute.collection.items
                        .filter(i => i.image && i.image.url)
                        .reduce((acc, i) => {
                        acc[i.key] = i.image.url.replace('coreo.imgix.net/itemImages/', 'static.coreo.app/_i/itemImages/');
                        return acc;
                    }, {});
                }
            }
            if (!this.asyncError) {
                this.formGeoJSONAndRender(sourceIndex);
            }
        });
    }
    formGeoJSONAndRender(sourceIndex) {
        const geojson = {
            type: "FeatureCollection",
            features: this.recordData[sourceIndex].map((singleRecordData) => {
                const feature = {
                    type: "Feature",
                    geometry: {
                        type: "Point",
                        coordinates: singleRecordData.geometry.coordinates
                    },
                    properties: {
                        id: singleRecordData.id,
                        iconSource: singleRecordData.iconSource
                    }
                };
                return feature;
            })
        };
        if (this.map.loaded()) {
            this.renderRecordData(geojson, sourceIndex);
        }
        else {
            this.map.on("load", () => this.renderRecordData(geojson, sourceIndex));
        }
    }
    loadSourceIcon(sourceIdx) {
        const fallbackIcon = 'https://i.imgur.com/MK4NUzI.png';
        const source = this.sources[sourceIdx];
        if (source.iconSourceType === 'default' || source.iconSourceType === 'url') {
            const imageId = `source-marker-${sourceIdx}`;
            this.addIconToMap(imageId, source.iconSource || fallbackIcon);
            return imageId;
        }
        else if (source.iconSourceType === 'survey') {
            Object.keys(this.iconData[sourceIdx]).forEach(key => {
                const url = this.iconData[sourceIdx][key] + '?w=42&h=42&fit=crop&mask=ellipse&bg=0FFF&maskbg=0FFF&fm=png';
                this.addIconToMap(key, url);
            });
            return ["get", "iconSource"];
        }
        else {
            const imageId = `source-marker-${sourceIdx}`;
            this.addIconToMap(imageId, fallbackIcon);
            console.warn('Unknown icon source type');
            return imageId;
        }
    }
    renderRecordData(geojson, sourceIndex) {
        this.map.resize();
        if (sourceIndex === 0) {
            this.configureMap();
        }
        const imageSource = this.loadSourceIcon(sourceIndex);
        this.map.addSource(("recordData" + sourceIndex), {
            type: "geojson",
            data: geojson,
            cluster: true,
            clusterMaxZoom: 10,
            clusterRadius: 30
        });
        if (this.pointStyle === 'circle') {
            this.map.addLayer({
                id: "unclustered-point" + sourceIndex,
                type: "circle",
                source: "recordData" + sourceIndex,
                filter: ["!has", "point_count"],
                paint: {
                    "circle-color": "#11b4da",
                    "circle-radius": 5,
                    "circle-stroke-width": 1,
                    "circle-stroke-color": "#fff"
                }
            });
        }
        if (this.pointStyle === 'marker') {
            this.map.addLayer({
                id: "marker" + sourceIndex,
                type: "symbol",
                source: "recordData" + sourceIndex,
                filter: ["!has", "point_count"],
                layout: {
                    "icon-image": imageSource,
                    "icon-ignore-placement": true,
                    "icon-allow-overlap": true,
                    "icon-anchor": "bottom"
                }
            });
        }
        this.map.addLayer({
            id: "clusters" + sourceIndex,
            type: "circle",
            source: "recordData" + sourceIndex,
            filter: ["has", "point_count"],
            paint: {
                "circle-color": [
                    "step",
                    ["get", "point_count"],
                    "#51bbd6",
                    100,
                    "#f1f075",
                    1000,
                    "#f28cb1"
                ],
                "circle-radius": [
                    "step",
                    ["get", "point_count"],
                    20,
                    100,
                    25,
                    1000,
                    30
                ]
            }
        });
        this.map.addLayer({
            "id": "cluster-count" + sourceIndex,
            "type": "symbol",
            "source": "recordData" + sourceIndex,
            "layout": {
                "text-field": "{point_count}",
                "text-font": [
                    "DIN Offc Pro Medium",
                    "Arial Unicode MS Bold"
                ],
                "text-size": 12
            }
        });
        this.map.on('mouseenter', 'clusters' + sourceIndex, () => {
            this.map.getCanvas().style.cursor = 'pointer';
        });
        this.map.on('mouseleave', 'clusters' + sourceIndex, () => {
            this.map.getCanvas().style.cursor = '';
        });
        this.map.on('click', 'clusters' + sourceIndex, (event) => {
            const features = this.map.queryRenderedFeatures(event.point, { layers: ['clusters' + sourceIndex] });
            const clusterId = features[0].properties.cluster_id;
            const source = this.map.getSource('recordData' + sourceIndex);
            source.getClusterExpansionZoom(clusterId, (error, zoom) => {
                if (error)
                    return;
                const newZoom = zoom === this.lastZoomLevel ? zoom + 1 : zoom;
                this.lastZoomLevel = newZoom;
                const geometry = features[0].geometry;
                this.map.easeTo({
                    center: geometry.coordinates,
                    zoom: newZoom
                });
            });
        });
        this.map.on('zoomend', (_event) => {
            this.lastZoomLevel = this.map.getZoom();
        });
        if (this.pointStyle === 'circle') {
            this.map.on('mouseenter', 'unclustered-point' + sourceIndex, () => {
                this.map.getCanvas().style.cursor = 'pointer';
            });
            this.map.on('mouseleave', 'unclustered-point' + sourceIndex, () => {
                this.map.getCanvas().style.cursor = '';
            });
            this.map.on('click', 'unclustered-point' + sourceIndex, (event) => {
                const features = this.map.queryRenderedFeatures(event.point, { layers: ['unclustered-point' + sourceIndex] });
                if (!features.length)
                    return;
                this.onPointClickedHandler(features[0], sourceIndex);
            });
        }
        if (this.pointStyle === 'marker') {
            this.map.on('mouseenter', 'marker' + sourceIndex, () => {
                this.map.getCanvas().style.cursor = 'pointer';
            });
            this.map.on('mouseleave', 'marker' + sourceIndex, () => {
                this.map.getCanvas().style.cursor = '';
            });
            this.map.on('click', 'marker' + sourceIndex, (event) => {
                const features = this.map.queryRenderedFeatures(event.point, { layers: ['marker' + sourceIndex] });
                if (!features.length)
                    return;
                this.onPointClickedHandler(features[0], sourceIndex);
            });
        }
    }
    addIconToMap(id, url) {
        return new Promise((resolve, reject) => {
            this.map.loadImage(url, (err, image) => {
                if (err) {
                    reject(err);
                }
                else {
                    this.map.addImage(id, image);
                    resolve();
                }
            });
        });
    }
    async configureMap() {
        if (this.showMapControls) {
            this.map.addControl(new mapboxgl.NavigationControl());
        }
        if (this.enableCtrl) {
            const mapElement = this.map.getCanvasContainer();
            const scrollHandler = (e) => {
                if (e.metaKey || e.ctrlKey) {
                    this.showCtrlBackdrop = false;
                    this.map.scrollZoom.enable();
                    this.map.scrollZoom.onWheel(e);
                    this.map.scrollZoom.disable();
                }
                else {
                    if (!this.showCtrlBackdrop && !this.showCtrlBackdropTimeout) {
                        this.showCtrlBackdrop = true;
                        this.showCtrlBackdropTimeout = true;
                        setTimeout(() => {
                            this.showCtrlBackdrop = false;
                            this.showCtrlBackdropTimeout = false;
                        }, 1000);
                    }
                }
            };
            mapElement.addEventListener('wheel', scrollHandler, false);
            mapElement.addEventListener('mousewheel', scrollHandler, false);
        }
    }
    render() {
        const divStyle = {
            position: 'absolute',
            top: '0',
            bottom: '0',
            left: '0',
            width: '100%'
        };
        const backdropStyle = {
            display: this.showCtrlBackdrop ? 'flex' : 'none',
            position: 'absolute',
            top: '0',
            bottom: '0',
            left: '0',
            width: '100%'
        };
        return [
            h("div", { style: divStyle, id: this.mapUUID }),
            h("div", { style: backdropStyle, class: "zoom-ctrl-backdrop", onWheel: (event) => this.ctrlBackdropHandler(event) },
                h("p", null, "Use Ctrl/Cmd + scroll to zoom the map"))
        ];
    }
    static get is() { return "coreo-mapbox"; }
    static get properties() { return {
        "apiCtrl": {
            "connect": "coreo-api"
        },
        "asyncError": {
            "state": true
        },
        "enableCtrl": {
            "type": Boolean,
            "attr": "enable-ctrl"
        },
        "iconData": {
            "state": true
        },
        "lastZoomLevel": {
            "state": true
        },
        "limit": {
            "type": Number,
            "attr": "limit"
        },
        "map": {
            "state": true
        },
        "mapboxToken": {
            "type": String,
            "attr": "mapbox-token"
        },
        "mapStyle": {
            "type": String,
            "attr": "map-style"
        },
        "mapUUID": {
            "state": true
        },
        "pointStyle": {
            "type": String,
            "attr": "point-style"
        },
        "projectId": {
            "type": Number,
            "attr": "project-id"
        },
        "recordData": {
            "state": true
        },
        "resizeMap": {
            "method": true
        },
        "showCtrlBackdrop": {
            "state": true
        },
        "showCtrlBackdropTimeout": {
            "state": true
        },
        "showMapControls": {
            "type": Boolean,
            "attr": "show-map-controls"
        },
        "sources": {
            "type": "Any",
            "attr": "sources"
        },
        "syncError": {
            "state": true
        }
    }; }
    static get events() { return [{
            "name": "onPointClicked",
            "method": "onPointClicked",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get style() { return "/**style-placeholder:coreo-mapbox:**/"; }
}
