/*! Built with http://stenciljs.com */
import { h } from '../coreo.core.js';

class CoreoError extends Error {
    constructor(msg, code) {
        super(msg);
        this.code = code;
    }
}
const handleFetchErrors = (response) => {
    if (!response.ok) {
        return response.json().then(data => {
            if (data.msg && data.code) {
                throw new CoreoError(data.msg, data.code);
            }
            throw Error(response.statusText);
        });
    }
    return response.json();
};
class CoreoAPIService {
    constructor(token, src = 'https://api.coreo.io', graphqlEndpoint = '/graphql') {
        this.token = token;
        this.src = src;
        this.graphqlEndpoint = graphqlEndpoint;
    }
    setToken(token) {
        this.token = token;
    }
    getToken() {
        return this.token;
    }
    setSrc(src) {
        this.src = src;
    }
    getSrc() {
        return this.src;
    }
    graphql(query, operation = 'query') {
        const options = {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                query: `${operation}{ ${query} }`
            })
        };
        if (this.token) {
            options.headers.Authorization = `JWT ${this.token}`;
        }
        return fetch(`${this.src}${this.graphqlEndpoint}`, options).then(handleFetchErrors);
    }
    post(path, body, isJson = false) {
        const options = {
            method: 'POST',
            body,
            headers: {
                'Authorization': `JWT ${this.token}`,
            }
        };
        if (isJson) {
            options.headers['Content-Type'] = 'application/json;charset=UTF-8';
        }
        return fetch(`${this.src}${path}`, options).then(handleFetchErrors);
    }
    get(path) {
        const options = {
            method: 'GET',
            headers: {
                'Authorization': `JWT ${this.token}`
            }
        };
        return fetch(`${this.src}${path}`, options).then(handleFetchErrors);
    }
}

class CoreoApiComponent {
    constructor() {
        this.src = 'https://api.coreo.io';
    }
    async create() {
        if (!this.api) {
            this.api = new CoreoAPIService(this.token, this.src);
        }
        return Promise.resolve(this.api);
    }
    static get is() { return "coreo-api"; }
    static get properties() { return {
        "create": {
            "method": true
        },
        "src": {
            "type": String,
            "attr": "src"
        },
        "token": {
            "type": String,
            "attr": "token"
        }
    }; }
}

class CoreoRecordImageComponent {
    constructor() {
        this.alt = "";
    }
    validateRecordId(newValue, _oldValue) {
        const isEmpty = typeof newValue == null;
        if (isEmpty) {
            throw new Error("<coreo-record-image /> ERROR: 'record-id' is a required property");
        }
    }
    validatePath(newValue, _oldValue) {
        const isEmpty = typeof newValue == null;
        if (isEmpty) {
            throw new Error("<coreo-record-image /> ERROR: 'path' is a required property");
        }
    }
    componentWillLoad() {
        this.apiCtrl.create().then(api => {
            this.src = api.getSrc();
        });
    }
    render() {
        if (!this.src) {
            return;
        }
        let image = null;
        if (this.recordId && this.path) {
            const source = `${this.src}/records/${this.recordId}/images/${this.path}`;
            image = h("img", { class: "coreo-record-image", src: source, alt: this.alt });
        }
        return image;
    }
    static get is() { return "coreo-record-image"; }
    static get properties() { return {
        "alt": {
            "type": String,
            "attr": "alt"
        },
        "apiCtrl": {
            "connect": "coreo-api"
        },
        "el": {
            "elementRef": true
        },
        "path": {
            "type": String,
            "attr": "path",
            "watchCallbacks": ["validatePath"]
        },
        "recordId": {
            "type": Number,
            "attr": "record-id",
            "watchCallbacks": ["validateRecordId"]
        },
        "src": {
            "state": true
        }
    }; }
    static get style() { return ".coreo-record-image{height:auto;max-width:100%}"; }
}

export { CoreoApiComponent as CoreoApi, CoreoRecordImageComponent as CoreoRecordImage };
